var mongoose = require('mongoose')
var should = require('should')
var FeedSchema = require('./feed')
var mongoomise = require('../src/main')
var Feed = mongoose.model('Feed', FeedSchema)
var ObjectId = mongoose.Types.ObjectId

mongoose.connect('mongodb://localhost/mongoomise')

module.exports = function(Promise){
	mongoomise.promisifyAll(mongoose, Promise)

	var document = {_id: new ObjectId(), text:'mongoomise'}

	describe('.countAsync', function(){
		it('should return a number', function(done){
			Feed.countAsync().then(function(total){
				total.should.be.type('number')
				done()
			})
		})
	})

	describe('.createAsync', function(){
		it('should return a document with target id', function(done){
			Feed.createAsync(document).then(function(doc){
				doc._id.toString().should.equal(document._id.toString())
				// validate pre save hooks
				doc.create_time.should.be.instanceof(Date)
				done()
			})
		})
	})

	describe('.distinctAsync', function(){
		it('should return an array', function(done){
			Feed.distinctAsync('text').then(function(results){
				results.should.be.instanceof(Array)
				done()
			})
		})
	})

	describe('.findAsync', function(){
		it('should return documents', function(done){
			Feed.findAsync().then(function(results){
				results.should.be.instanceof(Array)
				done()
			})
		})
	})

	describe('.updateAsync', function(){
		it('should return affected number', function(done){
			Feed.updateAsync({}, {pid:document._id}).then(function(results){
				results.should.be.instanceof(Array)
				results[0].should.be.type('number')
				done()
			})
		})
	})

	describe('.populateAsync', function(){
		it('should populate a doc', function(done){
			Feed.populateAsync({pid:document._id}, {path:'pid'}).then(function(doc){
				doc.pid._id.toString().should.equal(document._id.toString())
				done()
			})
		})
	})

	describe('.customAsync', function(){
		it('should support invoke custom static method', function(done){
			var input = Math.random()
			Feed.customAsync(input).then(function(output){
				input.should.equal(output)
				done()
			})
		})
	})

	// handle error
	describe('error', function(){
		it('should be threw on saving', function(done){
			var promise = new Feed(document).saveAsync()
			if(promise.catch){
				return promise.catch(function(err){
					err.should.be.instanceof(Error)
					done()
				})
			}else if(promise.error){
				return promise.error(function(err){
					err.should.be.instanceof(Error)
					done()
				})
			}
		})

		it('should be threw in pre save', function(done){
			var promise = new Feed({text:''}).saveAsync()
			if(promise.catch){
				return promise.catch(function(err){
					err.message.should.equal('error in pre save')
					done()
				})
			}else if(promise.error){
				return promise.error(function(err){
					err.message.should.equal('error in pre save')
					done()
				})
			}
		})

		it('should be threw in default validation', function(done){
			var promise = new Feed({pid: 'mongo'}).saveAsync()
			if(promise.catch){
				return promise.catch(function(err){
					err.constructor.name.should.equal('CastError')
					done()
				})
			}else if(promise.error){
				return promise.error(function(err){
					err.constructor.name.should.equal('CastError')
					done()
				})
			}
		})
	})

	describe('createConnection', function(){
		it('should support multiple connection', function(done){
			var conn = mongoose.createConnection('mongodb://localhost/mongoomise1')
			var Feed2 = conn.model('Feed2', FeedSchema)
			mongoomise.promisifyAll(conn, Promise)
			Feed2.customAsync(1).then(function(value){
				value.should.equal(1)
				return mongoose.model('Feed2').countAsync()
			}).then(function (value) {
				value.should.equal(0)
				done()
			})
		})
	})
}
